#!/usr/bin/perl
use Math::Trig;

############## set your paths for TMalign and stride ########## 
$tmalign="";
$stride="";
# examples #
$tmalign="/home/jinrui/bin/tmfold/TMalign";
$stride="/home/jinrui/bin/tmfold/stride";
# examples #


if(@ARGV<=1){usage();exit;}

for(my $i=0; $i<@ARGV; ++$i) {
    if($ARGV[$i] eq "-h") { usage(); exit; }
    if($ARGV[$i] eq "--help") { usage(); exit; }
    elsif($ARGV[$i] eq "-p1" && $i+1<@ARGV) { $pdbA =  $ARGV[++$i];}
    elsif($ARGV[$i] eq "-p2" && $i+1<@ARGV) { $pdbB =  $ARGV[++$i];}
    else {usage();exit;}
}

sub usage {
    print STDERR "$0";
    print STDERR " -p1 structure file1";
    print STDERR " -p2 structure file2\n";
}

############# secodnary structure calculation ################
$pdbA_ss = &get_ss($pdbA);
$pdbB_ss = &get_ss($pdbB);

@tmpss=split(/\t+/,$pdbA_ss);
$pdb=$tmpss[0];
$pdb_nn_hash{$pdb}=$tmpss[1]; # number of residues in coil region at N terminal
$pdb_cc_hash{$pdb}=$tmpss[2]; # number of residues in coil region at C terminal

@tmpss=split(/\t+/,$pdbB_ss);
$pdb=$tmpss[0];
$pdb_nn_hash{$pdb}=$tmpss[1];
$pdb_cc_hash{$pdb}=$tmpss[2];

############# tmscore calculation by TMalign ################
$tmalign_rst = &get_alignment($pdbA,$pdbB);

@tmpts=split(/\t+/,$tmalign_rst);
$pdb1=$tmpts[0];
$pdb2=$tmpts[1];
$l1=$tmpts[3]; # length of protein1
$l2=$tmpts[4]; # length of protein2

# number of coil residues at N-,C-terminals
$chop1_n=$pdb_nn_hash{$pdb1};
$chop1_c=$pdb_cc_hash{$pdb1};
$chop2_n=$pdb_nn_hash{$pdb2};
$chop2_c=$pdb_cc_hash{$pdb2};

# number of unaligned residues at N-,C-terminals
$tms_N1=$tmpts[5];
$tms_N2=$tmpts[6];
$tms_C1=$tmpts[7];
$tms_C2=$tmpts[8];

# number of unaligned and coil residues at N-,C-terminals
$chop1n=($chop1_n<$tms_N1)?$chop1_n:$tms_N1;
$chop2n=($chop2_n<$tms_N2)?$chop2_n:$tms_N2;
$chop1c=($chop1_c<$tms_C1)?$chop1_c:$tms_C1;
$chop2c=($chop2_c<$tms_C2)?$chop2_c:$tms_C2;

# number of resiudes can be chopped
$chop1=($chop1n+$chop1c);
$chop2=($chop2n+$chop2c);

# new length used in normalizing TM-score
$l1_new=$l1-$chop1;
$l2_new=$l2-$chop2;
$short_new=($l1_new<$l2_new)?$l1_new:$l2_new;
$short_old=($l1<$l2)?$l1:$l2;

# new TM-score
$tms_old=$tmpts[2];
$tms_new=$tmpts[2]*$short_old/$short_new;
if($tms_new>=1){
    $tms_new=1;
}
$tms_new=sprintf ("%.4f",$tms_new);
$tms_old=sprintf ("%.4f",$tms_old);

################## statistical analysis of tmscore ###############
# set parameters of posterior probability and EVD
$a1_scop=-0.0071735;
$a2_scop=0.99803;
$x0_scop=0.57917;
$dx_scop=0.048934;

$a1_cath=-0.004103;
$a2_cath=0.99217;
$x0_cath=0.52695;
$dx_cath=0.04688;

$u=0.1512;
$sigma=0.0242;

# pvalue and posterior probability calculation 
$pvalue=1-exp(0-exp(($u-$tms_new)/$sigma));
$prob_scop=$a2_scop+($a1_scop-$a2_scop)/(1+exp(($tms_new-$x0_scop)/$dx_scop));
$prob_cath=$a2_cath+($a1_cath-$a2_cath)/(1+exp(($tms_new-$x0_cath)/$dx_cath));
if($prob_scop<0){$prob_scop=0;}
if($prob_scop>1){$prob_scop=1;}
if($prob_cath<0){$prob_cath=0;}
if($prob_cath>1){$prob_cath=1;}

if($pvalue<0){$pvalue=0;}
if($pvalue>1){$pvalue=1;}
$prob_scop=sprintf ("%.4f",$prob_scop);
$prob_cath=sprintf ("%.4f",$prob_cath);
$pvalue=sprintf ("%.4f",$pvalue);

############################ output result ###################################
print "Proteins in Comparison:\n";
print "Structure 1: $pdb1\n";
print "Structure 2: $pdb2\n\n";

print "Similarity Scores:\n";
print "TM-score after  Normalization:                             $tms_new\n";
print "TM-score before Normalization:                             $tms_old\n\n";

print "Statistical Scores:\n";
print "Probability of sharing same fold (SCOP):                   $prob_scop\n";
print "Probability of sharing same topolgy (CATH):                $prob_cath\n";
print "P-value (Significance of the alignment):                   $pvalue\n\n";
print "$alignment";
print "//\n";

exit();

sub get_alignment{
    my ($pdbA, $pdbB)=@_; 
    my $k=0;

    my @result=`$tmalign $pdbA $pdbB -b`;
    chomp(@result);
    $result[8]=~ m/Chain 1:[\d\D\w\W\s]{5,}Size=([0-9\s]{2,5})\s*/;
    my $length1=$1;
    $result[9]=~ m/Chain 2:[\d\D\w\W\s]{5,}Size=([0-9\s]{2,5})\s*/;
    my $length2=$1;
        
    $result[11]=~ m/Aligned length=([0-9\.\s]{2,5}),/;
    my $aligned=$1;
    $result[11]=~ m/Aligned length=[0-9\.\s]{2,5}[\d\D\w\W\s]{5,}TM-score=([0-9\.\s]{2,}),/;
    my $tmscore=$1;
    
    my $ch1=$result[20];
    my $dot=$result[21];
    my $ch2=$result[22];
    $ch1 =~ s/\s+//g;
    $ch2 =~ s/\s+//g; 
    my @tmp1=split(//,$ch1);
    my @tmp2=split(//,$ch2);
    my @tmpD=split(//,$dot);
    my $N1=0; 
    my $N2=0;
    my $C1=0; 
    my $C2=0;
    for($k=0;$k<@tmp1;$k++){ 
	if($tmpD[$k]=~/\.|:/){ 
	    last; 
	} 
	if($tmp1[$k]=~/[A-Z]/){ 
	    $N1++; 
	} 
	if($tmp2[$k]=~/[A-Z]/){ 
	    $N2++; 
	}
    } 
    
    for($k=@tmp1;$k>=0;$k--){ 
	if($tmpD[$k]=~/\.|:/){ 
	    last;
	} 
	if($tmp1[$k]=~/[A-Z]/){ 
	    $C1++;
	} 
	if($tmp2[$k]=~/[A-Z]/){ 
	    $C2++;
	} 
    } 
    
    undef $alignment;
    for($k=13;$k<@result;$k++){
	$alignment.= $result[$k]."\n";
    }
    return "$pdbA\t$pdbB\t$tmscore\t$length1\t$length2\t$N1\t$N2\t$C1\t$C2";
}

sub get_ss{
    my ($pdb) = @_;
    my @sec;
    my $k;
    my $q;
    my $j;
    my @lines=`$stride $pdb 2> /dev/null`;
    chomp(@lines);
    for($j=0;$j<@lines;$j++){
	my $line=$lines[$j];
	if($line=~/^ASG/){
	    $k++;
	    $sec_tmp=substr($line,24,1);
	    $sec[$k]=1 if($sec_tmp eq "H");
	    $sec[$k]=2 if($sec_tmp eq "E");
	    $sec[$k]=3 if($sec_tmp ne "E" && $sec_tmp ne "H");
	}
    }
    my $chl=@sec;
    
    my $N1=0;
    my $C1=0;
    for($q=1;$q<$chl;$q++){    	
	if($sec[$q]!=3){
	    last;
	}
	$N1++;
    }
    for($q=($chl-1);$q>=1;$q--){    	
	if($sec[$q]!=3){
	    last;
	}
	$C1++;
    }
    my $tot=$N1+$C1;
    return "$pdb\t$N1\t$C1\t$tot";
}
